@@ -296,8 +296,7 @@ Syntax: var dfa-expr &body forms"
 	  "")
 	 (original-buffer		; Buffer (entity) where lit started
 	  (current-buffer))
-	 temp
-	 )
+	 temp)
     (cond
      ((or (sgml-parse-delim "LIT")
 	  (setq lita (sgml-parse-delim "LITA")))
@@ -322,8 +321,7 @@ Syntax: var dfa-expr &body forms"
 				     (if lita
 					 (sgml-skip-upto ("LITA" "PERO" "CRO"))
 				       (sgml-skip-upto ("LIT" "PERO" "CRO")))
-				     (point)))))))
-	)
+				     (point))))))))
       value))))
 
 (defun sgml-check-parameter-literal ()
