@@ -1,5 +1,5 @@
 ;;; psgml.el --- SGML-editing mode with parsing support
-;; $Id: psgml.el,v 1.10 2000/10/06 08:38:13 youngs Exp $
+;; $Id: psgml.el,v 1.12 2001/01/14 04:08:23 youngs Exp $
 
 ;; Copyright (C) 1993-1999 Lennart Staflin
 ;; Copyright (C) 1992 Free Software Foundation, Inc.
@@ -52,7 +52,7 @@
 
 ;;; Code:
 
-(defconst psgml-version "1.2.1"
+(defconst psgml-version "1.2.2"
   "Version of psgml package.")
 
 (defconst psgml-maintainer-address "lenst@lysator.liu.se")
@@ -83,6 +83,10 @@
 
 ;;; User settable options:
 
+(defvar sgml-insert-defaulted-attributes nil
+  "*Controls whether defaulted attributes (not #FIXED) are inserted explicitly
+or not. nil means don't insert, t means insert.")
+
 (defgroup sgml nil
   "Standard Generalized Markup Language"
   :group 'languages)
@@ -320,6 +324,10 @@ and legal tags beyond omittable end tags."
   :type 'boolean
   :group 'psgml)
 
+(defvar sgml-omittag-transparent (not sgml-balanced-tag-edit)
+  "*If non-nil, will show legal tags inside elements with omitable start tags
+and legal tags beyond omitable end tags.")
+
 (defcustom sgml-leave-point-after-insert nil
   "*If non-nil, the point will remain after inserted tag(s).
 If nil, the point will be placed before the inserted tag(s)."
@@ -610,7 +618,7 @@ format control string instead of the defaults."
   :group 'psgml)
 
 (defvar sgml-validate-error-regexps
-  '((".*:\\(.+\\):\\([0-9]+\\):\\([0-9]+\\):[EX]: " 1 2 3)
+  '((".*:\\(.+\\):\\([0-9]+\\):\\([0-9]+\\):[EXW]: " 1 2 3)
     ("\\(error\\|warning\\) at \\([^,]+\\), line \\([0-9]+\\)" 2 3)
     ("\n[a-zA-Z]?:?[^0-9 \n\t:]+:[ \t]*\\([^ \n\t:]+\\):\
 \\([0-9]+\\):\\(\\([0-9]+\\)[: \t]\\)?" 1 2 4))
@@ -897,6 +905,14 @@ as that may change."
 	 'sgml-tag-region-if-active
          'sgml-use-text-properties
 	 ))))
+
+;;;; SGML mode: syntax table
+
+(defvar sgml-mode-syntax-table
+  (let ((s (copy-syntax-table text-mode-syntax-table)))
+    (modify-syntax-entry ?< "."  s)
+    (modify-syntax-entry ?> "."  s)
+    s))
 
 
 ;;;; SGML mode: keys and menus
@@ -954,7 +970,7 @@ as that may change."
 (define-key sgml-mode-map "\e\C-k"   'sgml-kill-element)
 (define-key sgml-mode-map "\e\C-@"   'sgml-mark-element)
 ;;(define-key sgml-mode-map [?\M-\C-\ ] 'sgml-mark-element)
-(define-key sgml-mode-map "\e\C-h"   'sgml-mark-current-element)
+(define-key sgml-mode-map [(meta control h)] 'sgml-mark-current-element)
 (define-key sgml-mode-map "\e\C-t"   'sgml-transpose-element)
 (define-key sgml-mode-map "\M-\t"    'sgml-complete)
 
@@ -1115,12 +1131,15 @@ as that may change."
     (when (or (null last-values)
               (sgml-any-option-changed (cdr last-values)
                                        option-vars))
-      (easy-menu-change (list sgml-menu-name) menuname
-                      (nconc (sgml-options-menu-items option-vars)
-                             (if save-func
-                                 (list "---"
-                                       (vector (format "Save %s" menuname)
-                                               save-func t)))))
+      (condition-case err
+          (easy-menu-change (list sgml-menu-name) menuname
+                            (nconc (sgml-options-menu-items option-vars)
+                                   (if save-func
+                                       (list "---"
+                                             (vector (format "Save %s" menuname)
+                                                     save-func t)))))
+        (error
+         (message "Error in update menu: %s" err)))
       (unless last-values
         (setq last-values (cons menuname nil))
         (push last-values sgml-last-options-menu-values))
@@ -1293,7 +1312,7 @@ All bindings:
   (set (make-local-variable 'paragraph-start)
        paragraph-separate)
 
-  (set-syntax-table text-mode-syntax-table)
+  (set-syntax-table sgml-mode-syntax-table)
   (make-local-variable 'comment-start)
   (setq comment-start "<!-- ")
   (make-local-variable 'comment-end)
@@ -1350,6 +1369,7 @@ All bindings:
   (setq sgml-minimize-attributes nil)
   (setq sgml-always-quote-attributes t)
   (setq sgml-validate-command "nsgmls -wxml -s %s %s")
+  ;; FIXME: why conditional?
   (unless sgml-declaration
     (make-local-variable 'sgml-declaration)
     (setq sgml-declaration sgml-xml-declaration)))
@@ -1562,6 +1582,7 @@ and move to the line in the SGML document that caused it."
 	  nil nil)
 (autoload 'sgml-indent-line "psgml-edit" nil)
 (autoload 'sgml-element-endable-p "psgml-edit" nil)
+(autoload 'sgml-do-set-option "psgml-edit" nil)
 
 ;;; Generated by sgml-build-autoloads
 
